-- SPDX-FileCopyrightText: 2023 Sam Nystrom <sam@samnystrom.dev>
-- SPDX-License-Identifier: GPL-3.0-only

create table if not exists file(
    id integer primary key,
    name text unique not null,
    type text not null,
    content blob
);

create unique index if not exists file_name on file(name);

create table if not exists nodegroup(
    id integer primary key,
    name text unique not null
);

create unique index if not exists nodegroup_name on nodegroup(name);

create table if not exists nodegroup_input(
    id integer primary key,
    nodegroup_id integer references nodegroup on delete cascade not null,
    name text not null,
    idx integer not null,
    default_value,
    control_type text,
    control_info text -- JSON
);

create index if not exists nodegroup_input_name on nodegroup_input(nodegroup_id, name);
create index if not exists nodegroup_input_idx on nodegroup_input(nodegroup_id, idx);

create table if not exists nodegroup_output(
    id integer primary key,
    nodegroup_id integer references nodegroup on delete cascade not null,
    name text not null,
    idx integer not null
);

create index if not exists nodegroup_output_name on nodegroup_output(nodegroup_id, name);
create index if not exists nodegroup_output_idx on nodegroup_output(nodegroup_id, idx);

create table if not exists node(
    id integer primary key,
    nodegroup_id integer references nodegroup not null,
    subgroup_id integer references nodegroup,
    type text,
    x integer not null default 0,
    y integer not null default 0,
    w integer,
    h integer
);

create table if not exists node_input(
    id integer primary key,
    node_id integer references node on delete cascade not null,
    name text not null,
    idx integer not null,
    control_type text,
    control_info text, -- JSON
    value
);

create table if not exists node_output(
    id integer primary key,
    node_id integer references node on delete cascade not null,
    name text not null,
    idx integer not null
);

create trigger if not exists node_groupinput_insert after insert on nodegroup_input begin
    insert into node_output (node_id, name, idx)
        select node.id, new.name, new.idx from node
        where node.type = 'Group Input' and node.nodegroup_id = new.nodegroup_id;
end;

-- TODO: group input update/delete and group output insert/update/delete

create table if not exists link(
    from_id integer references node_output on delete cascade not null,
    to_id integer references node_input on delete cascade not null,
    primary key(from_id, to_id)
);

create index if not exists link_back on link(to_id, from_id);

create table if not exists pane(
    id integer primary key,
    title text not null,
    description text not null,
    nodegroup_id integer references nodegroup on delete set null,
    x integer not null,
    y integer not null,
    w integer not null,
    h integer not null
);

create table if not exists pane_input(
    pane_id integer references pane not null,
    nodegroup_input_id integer references nodegroup_input not null,
    control_type text not null,
    control_info text, -- JSON
    value,
    primary key(pane_id, nodegroup_input_id)
);

create table if not exists selected_pane(
    id integer primary key check(id = 1),
    pane_id integer references pane on delete set null
);
insert or ignore into selected_pane values (1, null);

create table if not exists selected_nodegroup(
    id integer primary key check(id = 1),
    nodegroup_id integer references nodegroup on delete set null
);
insert or ignore into selected_nodegroup values (1, null);
